@@ -7,7 +7,7 @@
  *
  * Gnome-terminal is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * Gnome-terminal is distributed in the hope that it will be useful,
@@ -66,11 +66,12 @@ initial_tab_free (InitialTab *it)
 }
 
 static InitialWindow*
-initial_window_new (void)
+initial_window_new (guint source_tag)
 {
   InitialWindow *iw;
 
   iw = g_slice_new0 (InitialWindow);
+  iw->source_tag = source_tag;
 
   return iw;
 }
@@ -117,7 +118,7 @@ ensure_top_window (TerminalOptions *options)
 
   if (options->initial_windows == NULL)
     {
-      iw = initial_window_new ();
+      iw = initial_window_new (0);
       iw->tabs = g_list_append (NULL, initial_tab_new (NULL, FALSE));
       apply_defaults (options, iw);
 
@@ -155,7 +156,7 @@ add_new_window (TerminalOptions *options,
 {
   InitialWindow *iw;
 
-  iw = initial_window_new ();
+  iw = initial_window_new (0);
   iw->tabs = g_list_prepend (NULL, initial_tab_new (profile, is_id));
   apply_defaults (options, iw);
 
@@ -178,7 +179,7 @@ unsupported_option_callback (const gchar *option_name,
 }
 
 
-static gboolean
+static gboolean G_GNUC_NORETURN
 option_version_cb (const gchar *option_name,
                    const gchar *value,
                    gpointer     data,
@@ -187,7 +188,6 @@ option_version_cb (const gchar *option_name,
   g_print ("%s %s\n", _("GNOME Terminal"), VERSION);
 
   exit (EXIT_SUCCESS);
-  return FALSE;
 }
 
 static gboolean
@@ -496,11 +496,12 @@ option_load_save_config_cb (const gchar *option_name,
 
   if (options->config_file)
     {
-      g_set_error (error, 0, 0, "X"); /* FIXME */
+      g_set_error_literal (error, TERMINAL_OPTION_ERROR, TERMINAL_OPTION_ERROR_EXCLUSIVE_OPTIONS,
+                           "Options \"--load-config\" and \"--save-config\" are mutually exclusive");
       return FALSE;
     }
 
-  options->config_file = g_strdup (value);
+  options->config_file = terminal_util_resolve_relative_path (options->default_working_dir, value);
   options->load_config = strcmp (option_name, "--load-config") == 0;
   options->save_config = strcmp (option_name, "--save-config") == 0;
 
@@ -635,10 +636,6 @@ digest_options_callback (GOptionContext *context,
   TerminalOptions *options = data;
   InitialTab    *it;
 
-  /* make sure we have some window in case no options were given */
-  if (options->initial_windows == NULL)
-    it = ensure_top_tab (options);
-
   if (options->execute)
     {
       if (options->exec_argv == NULL)
@@ -661,11 +658,32 @@ digest_options_callback (GOptionContext *context,
   return TRUE;
 }
 
+/**
+ * terminal_options_parse:
+ * @working_directory: the default working directory
+ * @display_name: the default X display name
+ * @startup_id: the startup notification ID
+ * @env: the environment as variable=value pairs
+ * @remote_arguments: whether the caller is the factory process or not
+ * @ignore_unknown_options: whether to ignore unknown options when parsing
+ *   the arguments
+ * @argcp: (inout) address of the argument count. Changed if any arguments were handled
+ * @argvp: (inout) address of the argument vector. Any parameters understood by
+ *   the terminal #GOptionContext are removed
+ * @error: a #GError to fill in
+ * @...: a %NULL terminated list of extra #GOptionGroup<!-- -->s
+ *
+ * Parses the argument vector *@argvp.
+ *
+ * Returns: a new #TerminalOptions containing the windows and tabs to open,
+ *   or %NULL on error.
+ */
 TerminalOptions *
 terminal_options_parse (const char *working_directory,
                         const char *display_name,
                         const char *startup_id,
                         char **env,
+                        gboolean remote_arguments,
                         gboolean ignore_unknown_options,
                         int *argcp,
                         char ***argvp,
@@ -682,6 +700,7 @@ terminal_options_parse (const char *working_directory,
 
   options = g_slice_new0 (TerminalOptions);
 
+  options->remote_arguments = remote_arguments;
   options->default_window_menubar_forced = FALSE;
   options->default_window_menubar_state = TRUE;
   options->default_fullscreen = FALSE;
@@ -757,9 +776,22 @@ terminal_options_parse (const char *working_directory,
   return NULL;
 }
 
+/**
+ * terminal_options_merge_config:
+ * @options:
+ * @key_file: a #GKeyFile containing to merge the options from
+ * @source_tag: a source_tag to use in new #InitialWindow<!-- -->s
+ * @error: a #GError to fill in
+ *
+ * Merges the saved options from @key_file into @options.
+ *
+ * Returns: %TRUE if @key_file was a valid key file containing a stored
+ *   terminal configuration, or %FALSE on error
+ */
 gboolean
 terminal_options_merge_config (TerminalOptions *options,
                                GKeyFile *key_file,
+                               guint source_tag,
                                GError **error)
 {
   int version, compat_version;
@@ -770,7 +802,8 @@ terminal_options_merge_config (TerminalOptions *options,
 
   if (!g_key_file_has_group (key_file, TERMINAL_CONFIG_GROUP))
     {
-      g_set_error_literal (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND,
+      g_set_error_literal (error, TERMINAL_OPTION_ERROR,
+                           TERMINAL_OPTION_ERROR_INVALID_CONFIG_FILE,
                            _("Not a valid terminal config file."));
       return FALSE;
     }
@@ -782,7 +815,8 @@ terminal_options_merge_config (TerminalOptions *options,
       compat_version <= 0 ||
       compat_version > TERMINAL_CONFIG_COMPAT_VERSION)
     {
-      g_set_error_literal (error, 0 /* FIXME */, 0,
+      g_set_error_literal (error, TERMINAL_OPTION_ERROR,
+                           TERMINAL_OPTION_ERROR_INCOMPATIBLE_CONFIG_FILE,
                            _("Incompatible terminal config file version."));
       return FALSE;
     }
@@ -802,7 +836,7 @@ terminal_options_merge_config (TerminalOptions *options,
       if (!tab_groups)
         continue; /* no tabs in this window, skip it */
 
-      iw = initial_window_new ();
+      iw = initial_window_new (source_tag);
       initial_windows = g_list_append (initial_windows, iw);
       apply_defaults (options, iw);
 
@@ -861,6 +895,24 @@ terminal_options_merge_config (TerminalOptions *options,
   return TRUE;
 }
 
+/**
+ * terminal_options_ensure_window:
+ * @options:
+ *
+ * Ensure that @options will contain at least one window to open.
+ */
+void
+terminal_options_ensure_window (TerminalOptions *options)
+{
+  ensure_top_window (options);
+}
+
+/**
+ * terminal_options_free:
+ * @options:
+ *
+ * Frees @options.
+ */
 void
 terminal_options_free (TerminalOptions *options)
 {
@@ -982,7 +1034,7 @@ get_goption_context (TerminalOptions *options)
       0,
       G_OPTION_ARG_CALLBACK,
       option_geometry_callback,
-      N_("Set the window geometry from the provided X geometry specification; see the \"X\" man page for more information"),
+      N_("Set the window size; for example: 80x24, or 80x24+200+200 (ROWSxCOLS+X+Y)"),
       N_("GEOMETRY")
     },
     {
@@ -1049,7 +1101,7 @@ get_goption_context (TerminalOptions *options)
       0,
       G_OPTION_ARG_CALLBACK,
       option_zoom_callback,
-      N_("Set the terminalx's zoom factor (1.0 = normal size)"),
+      N_("Set the terminal's zoom factor (1.0 = normal size)"),
       N_("ZOOM")
     },
     { NULL, 0, 0, 0, NULL, NULL, NULL }
